/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WKUTTARESIDUAL_H
#define WKUTTARESIDUAL_H

#include "dart.h"

#include <vector>
#include <Eigen/Dense>

namespace dart
{

/**
 * @brief Formulation of Kutta residuals
 */
class DART_API KuttaResidual
{
public:
    // Picard
    static Eigen::MatrixXd buildFixed(KuttaElement const &ke, std::vector<double> const &phi);
    // Newton
    static Eigen::VectorXd build(KuttaElement const &ke, std::vector<double> const &phi);
    static Eigen::MatrixXd buildGradientFlow(KuttaElement const &ke, std::vector<double> const &phi);
    // Adjoint
    static std::tuple<Eigen::MatrixXd, Eigen::MatrixXd> buildGradientMesh(KuttaElement const &ke, std::vector<double> const &phi);
};

} // namespace dart
#endif // WKUTTARESIDUAL_H
